// ================================================================
// $Id: tgaimg.h,v 1.2 2005/11/30 15:32:51 havran Exp $
//
// tgaimg.h
//
//    The definition of image in TGA format - interior representation if float
//
// REPLACEMENT_STRING
//
// Initial coding by Vlastimil Havran, 2012

#ifndef __TGAIMG_H__
#define __TGAIMG_H__

#include <cstdio>
#include <iostream>

class CEnvironmentMap;

enum EMarkType
{
  EE_BRDFSamples = 0,
  EE_ENVMAPSamples = 1,  
};

// -----------------------------------------------------------------------------
// The TGA image class
class CTGAImage
{ /* Sun raster file header */
  int _width, _height;
  float *_buf;
  float *_maxAdr;
  float *_adr;
  enum {CHANNELS = 3};
  inline void ComputeAdr(int x, int y) {
    assert( (x>=0) && (x<_width));
    assert( (y>=0) && (y<_height));
    _adr = _buf + CHANNELS * (y*_width + x);
    assert(_adr < _maxAdr);
  }
public:
  // constructor and destructor
  CTGAImage() { _width = _height = 0; _buf = 0;}
  ~CTGAImage() { CleanUp(); }

  void CleanUp() { delete []_buf; _buf = 0; _maxAdr = 0;}
  
  int GetWidth() const { return _width;}
  int GetHeight() const { return _height;}
  
  void Init(int width, int height) {
    delete [] _buf;
    assert(width > 0);
    assert(height > 0);
    _width = width; _height = height;
    _buf = new float[_width * _height * CHANNELS];
    _maxAdr = &_buf[_width * _height * CHANNELS];
    assert(_buf);
    // make the image black
    memset(_buf, 0, _width * _height * CHANNELS * sizeof(float));
  }

  void SetPixel(int x, int y, float r, float g, float b) {
    ComputeAdr(x,y);
    *_adr = r; _adr++;
    *_adr = g; _adr++;
    *_adr = b;    
  }

  void SetPixelClip(int x, int y, float r, float g, float b) {
    if ((x < 0) || (x >= _width) || (y < 0) || (y >= _height))
      return;
    ComputeAdr(x,y);
    *_adr = r; _adr++;
    *_adr = g; _adr++;
    *_adr = b;    
  }

  void GetPixel(int x, int y, float &r, float &g, float &b) {
    ComputeAdr(x,y);
    r = *_adr; _adr++;
    g = *_adr; _adr++;
    b = *_adr;
  }
  // save to a file of a given name - return 0 on success
  int SaveToFile(const char *filename);

  // makes reinitialization and copy the data from here, using possibly multiplier
  void copyImageDataFromEM(CEnvironmentMap &hr, int mode, int mult = 1.0);  

  // make a mark at specific point in the image
  void MakeMark(int mx, int my, EMarkType typeMark, int marksize = 2);
  
  // draw a line between two points, no clipping possible
  void DrawLine(int x1, int y1, int x2, int y2,
		float red, float green, float blue);

};

inline int
CTGAImage::SaveToFile(const char *filename)
{
  int i, j;

  int channelNum = CHANNELS;
  int width = _width;
  int height = _height;

  // Only 3 or 4 channels in file are supported
  if (channelNum != 3 && channelNum != 4) {
    std::cerr << "Invalid number of channels: " << channelNum << '\n';
    std::cerr << "Only 3 or 4 channels in file are supported\n";
    return 1; // error
  }
  FILE *f = fopen(filename,"wb");
  if (!f)
    return 1; // file cannot be opened

  printf("Saving image (%d x %d) to a file %s \n", width, height, filename);

  fprintf(f,"%c", 0);           // CImage ID length (no image ID = 0)
  fprintf(f,"%c", 0);           // Colormap type (no colormap = 0)
  fprintf(f,"%c", 2);           // CImage type (uncompressed true-color = 2)
  // Color Map specification (0 - no colormap)
  fprintf(f,"%c%c%c%c%c",0,0,0,0,0);               
  fprintf(f,"%c%c", 0, 0);           // X image origin (0 - no offset)
  fprintf(f,"%c%c", 0, 0);           // Y image origin (0 - no offset)
  // CImage width
  fprintf(f,"%c%c", ((unsigned char)(width >> 0)), ((unsigned char)(width >> 8)));
  // CImage height
  fprintf(f,"%c%c", ((unsigned char)(height >> 0)), ((unsigned char)(height >> 8)));
  // Pixel depth (24 or 32 bpp: true-color)
  fprintf(f,"%c", (channelNum * 8));
                            
  if (channelNum == 4)         // CImage descriptor
    fprintf(f,"%c", 0x28);     // 0010X000 - 00 - unused
  else                         //            10 - left-to-right,top-to-bottom
    fprintf(f,"%c", 0x20);     //            0000 or 1000 - no alpha or
                               //                           1 byte of alpha
  // CImage ID not used
  // CImage colormap not used
  // CImage data follows (RGB triplets)
  fflush(f);

  for (j = 0; j < height; j++){
    for (i = 0; i < width; i++) {
      ComputeAdr(i, j);
      unsigned char val;

      float r,g,b;
      r = *_adr; _adr++;
      g = *_adr; _adr++;
      b = *_adr;
      
      if (channelNum >= 3) {
	val = (unsigned char) b; // Blue
	fprintf(f,"%c", val); // Blue
      }
      if (channelNum >= 2) {
        val = (unsigned char) g; // Green
        fprintf(f,"%c", val); // Green
      }
      if (channelNum >= 1) {
        val = (unsigned char) r; // Red
        fprintf(f,"%c", val); // Red
      }

      if (channelNum == 4) {
	_adr++; // alpha value should be in range 0.0-1.0
	float a = *_adr;
        fprintf(f,"%c", (unsigned char)(255.0 * a)); // Alpha
      }
    }
  }
  fflush(f);
  fprintf(f,"%c%c%c%c",0,0,0,0);  // no extension area - offset = 0
  fprintf(f,"%c%c%c%c",0,0,0,0); // no developers directory - offset = 0
  fprintf(f,"TRUEVISION-XFILE."); // new TGA signature
  fclose(f);
  return 0; // ok
}

inline void
CTGAImage::MakeMark(int mx, int my, EMarkType typeMark, int marksize)
{
  int width = GetWidth();
  int height = GetHeight();
  int x, y;
  
  float r, g, b;
  if (typeMark == EE_BRDFSamples) {
    r = g = b = 255.0; // white
  }
  else {
    // ENVMAP samples
    r = 255.0; g = b = 0.0; // red
  }
  // Make mark with correct color

  if (typeMark == EE_BRDFSamples) {
    // the horizontal direction
    if ((my >= 0) && (my < height)) {
      for (x = mx - marksize; x <= mx + marksize; x++) {
	if ((x < 0) || (x >= width))
	  continue;
	SetPixel(x, my, r, g, b);
      }
    }
    if ((mx >= 0) && (mx < width)) {
      for (y = my - marksize; y <= my + marksize; y++) {
	if ((y < 0) || (y >= height))
	  continue;
	SetPixel(mx, y, r, g, b);
      }
    }
  }
  else {
    int t;
    // from left bottom to right top corner
    for (t = - marksize; t <= marksize; t++) {
      x = mx + t;
      if ((x < 0) || (x >= width))
	continue;
      y = my + t;
      if ((y < 0) || (y >= height))
	continue;
      SetPixel(x, y, r, g, b);
    } // for t
    // from left top to right bottom corner
    for (t = - marksize; t <= marksize; t++) {
      x = mx + t;
      if ((x < 0) || (x >= width))
	continue;
      y = my - t;
      if ((y < 0) || (y >= height))
	continue;
      SetPixel(x, y, r, g, b);
    } // for t
  }

  return; // mark finished
}

// Trivial algorithm to render a line for purposes of visualization
inline void
CTGAImage::DrawLine(int x1, int y1, int x2, int y2,
		    float red, float green, float blue)
{
  if (fabs(x1 - x2) > fabs(y1 - y2)) {
    // the loop along the x-axis
    int deltax = (x2 - x1) > 0 ? 1: -1;
    for (int x = x1; x < x2; x++) {
      int y = y1 + (y2 - y1) * (x - x1) / (x2 - x1);
      SetPixelClip(x, y, red, green, blue);
    }
  }
  else {
    // the loop along the y-axis
    int deltay = (y2 - y1) > 0 ? 1 : -1;
    for (int y = y1; y < y2; y++) {
      int x = x1 + (x2 - x1) * (y - y1) / (y2 - y1);
      SetPixelClip(x, y, red, green, blue);
    }
  }
  
  return;
}

#endif // __TGAIMG_H__

