#include <cassert>
#include <cstdio>
#include <cmath>
#include <cstdlib>

#include "configg.h"
#include "AuxFuncs.h"
#include "AuxRND.h"
#ifdef __USE_MERSENNE_TWISTER
#include "mtwister.h"
#endif // __USE_MERSENNE_TWISTER

void
ConvertThetaPhiToXYZ(float theta, float phi, float xyz[])
{
  // Here we convert the angles to 3D vector
  xyz[0] = cos(phi) * sin(theta);
  xyz[1] = sin(phi) * sin(theta);
  xyz[2] = cos(theta);
}//--- ConvertThetaPhiToXYZ2 --------------------------------

// The method that converts the directional vector on the hemisphere to the 
// angle theta (from zenith) and phi angle (the azimuth)
void
ConvertXYZtoThetaPhi(float xyz[3], // input direction
		     float &theta, float &phi) // output direction theta and phi
{
  float mag = sqrt(xyz[0] * xyz[0] + xyz[1] * xyz[1]+ xyz[2] * xyz[2]);
  assert(mag > 1e-15f);
  mag = 1.0f/mag - 1e-5f;
  xyz[0] *= mag;
  xyz[1] *= mag;
  xyz[2] *= mag;
  assert(xyz[0] >= -1.f);
  assert(xyz[1] >= -1.f);
  assert(xyz[2] >= -1.f);
  assert(xyz[0] <= 1.f);
  assert(xyz[1] <= 1.f);
  assert(xyz[2] <= 1.f);

  // Standard conversion
  theta = acos(xyz[2]);
  if (theta > M_PI)
    theta = M_PI;
  phi = atan2(xyz[1], xyz[0]);

  if (phi < 0.f)
    phi += 2.f*M_PI;
  if (phi > 2.f * M_PI)
    phi -= 2.f*M_PI;

  assert( (phi >= 0.f) && (phi <= 2.f * M_PI));
  return;
}

void
ArbitraryNormal(const float N[3], float outN[3])
{
  float dist2 = N[0] * N[0] + N[1] * N[1]; // in X and Y
  if (dist2 > 0.001f) { // typical case
    float inv_size = 1.0f/sqrt(dist2);
    outN[0] = N[1] * inv_size;
    outN[1] = -N[0] * inv_size;
    outN[2] = 0.f; // N x (0,0,1)
    return;
  }
  // N is close to the the direction of Z axis
  float inv_size = 1.0/sqrt(N[2] * N[2] + N[0] * N[0]);
  outN[0] = -N[2] * inv_size;
  outN[1] = 0.f;
  outN[2] = N[0] * inv_size; // N x (0,1,0)
  return;
}


int invertMatrix(float* mat, float* im)
{
  float rmat[4][8];
  float a, b;
  int i, j, k;
  
  // initialize reduction matrix
  rmat[0][0] = mat[0];
  rmat[1][0] = mat[1];
  rmat[2][0] = mat[2];
  rmat[3][0] = mat[3];
  rmat[0][1] = mat[4];
  rmat[1][1] = mat[5];
  rmat[2][1] = mat[6];
  rmat[3][1] = mat[7];
  rmat[0][2] = mat[8];
  rmat[1][2] = mat[9];
  rmat[2][2] = mat[10];
  rmat[3][2] = mat[11];
  rmat[0][3] = mat[12];
  rmat[1][3] = mat[13];
  rmat[2][3] = mat[14];
  rmat[3][3] = mat[15];
  rmat[0][4] = 1.0f;
  rmat[1][4] = 0.0f;
  rmat[2][4] = 0.0f;
  rmat[3][4] = 0.0f;
  rmat[0][5] = 0.0f;
  rmat[1][5] = 1.0f;
  rmat[2][5] = 0.0f;
  rmat[3][5] = 0.0f;
  rmat[0][6] = 0.0f;
  rmat[1][6] = 0.0f;
  rmat[2][6] = 1.0f;
  rmat[3][6] = 0.0f;
  rmat[0][7] = 0.0f;
  rmat[1][7] = 0.0f;
  rmat[2][7] = 0.0f;
  rmat[3][7] = 1.0f;
  
  // perform reductions
  for(i=0; i<4; i++){
    a = rmat[i][i];
    if(a == 0.0f)  // matrix is singular, can't be reduced
      return 0;
    else{
      a = 1.0f / a;
      for(j=0; j<8; j++)
	rmat[i][j] = rmat[i][j] * a;
      for(k=0; k<4; k++){
	if((k-i) != 0){
	  b = rmat[k][i];
	  for(j=0; j<8; j++)
	    rmat[k][j] = rmat[k][j] - b * rmat[i][j];
	}
      }
    }
  }

  // extract the inverted matrix
  im[0] = rmat[0][4];
  im[1] = rmat[1][4];
  im[2] = rmat[2][4];
  im[3] = rmat[3][4];
  im[4] = rmat[0][5];
  im[5] = rmat[1][5];
  im[6] = rmat[2][5];
  im[7] = rmat[3][5];
  im[8] = rmat[0][6];
  im[9] = rmat[1][6];
  im[10] = rmat[2][6];
  im[11] = rmat[3][6];
  im[12] = rmat[0][7];
  im[13] = rmat[1][7];
  im[14] = rmat[2][7];
  im[15] = rmat[3][7];
  
  return 1;
}//--- invertMatrix -------------------------------------------------

