// ===================================================================
// $Id: basstr.cpp,v 1.5 2005/11/30 15:46:46 havran Exp $
//
// basic.cpp
//     Helper routines.
//
// REPLACEMENT_STRING
//
// Initial coding by Vlastimil Havran, 1999.

// GOLEM headers
#include "configg.h"
#include "basstr.h"

#include <string>

__BEGIN_SBRDFLIB_SRC

// Puts the specified number of spaces out to given stream. Used for
// indentation during CObject3D::Describe().
void
indent(ostream &app, int ind)
{
  int i;

  for (i = 0; i < ind; i++)
    app << ' ';
}

// prefix filename by a given directory name e.g. "aa.out","ref" makes
// "ref/aa.out", but "stats/aa.out","ref" makes "stats/ref/aa.out"
void
PrefixNameByDir(const string &filename, const string &dirname,
		string &outputName)
{
  outputName.resize(0);
  char dircat[2];
  dircat[0] = (char) DIRCAT;
  dircat[1] = 0x00;

  unsigned int pos = (unsigned int)filename.find_last_of(dircat[0]);

  //DEBUG << "pos = " << pos << endl;
  
  if (pos < filename.length()) { // current name contains some dir separators
    outputName.assign(filename, 0, pos-1);
    //DEBUG << "outputName=" << outputName.c_str()<< endl;
    outputName += dircat;
  }
  else
    pos = 0;

  //DEBUG << "outputName=" << outputName.c_str()<< endl;
    
  outputName.append(dirname);
  //DEBUG << "outputName=" << outputName.c_str()<< endl;
  outputName += dircat;
  //DEBUG << "outputName=" << outputName.c_str()<< endl;
  outputName.append(filename, pos, filename.length()-pos);
  //DEBUG << "outputName=" << outputName.c_str()<< endl;
}

// prefix filename extension by a given extPrefix
// for example the filename "data/tetra.rgb" and extPrefix 0102 makes
// the output filename "data/tetra0102.rgb"
// If no extension is found, then extPrefix is simply appended to the
// file name, e.g. for "balls" makes "balls0102"
void
PrefixExtensionByName(const string &filename, const string &extPrefix,
	  	      string &outputName)
{
  unsigned int pos = (unsigned int)filename.find_last_of('.');
  
  if (pos >= filename.length()) {
    outputName = filename;
    outputName.append(extPrefix); // just append
  }
  else {
    // the storage for extension of filename
    outputName.assign(filename, 0, pos);
    // append the extPrefix
    outputName.append(extPrefix);
    outputName.append(filename, pos, filename.length()-pos);
  }

  return;
}


// Removes path from filename,
// for example the filename "data/tetra.rgb" gets "tetra.rgb"
void
RemovePathFromFilename(const string &filename, string &outputName)
{
  unsigned int pos = (unsigned int)filename.find_last_of('/');
  int length = filename.length();
  if (pos >= length) {
    outputName = filename; // no path exists
  }
  else {
    // the storage for extension of filename
    outputName.assign(filename, pos+1, length-pos-1);
  }

  return;
}


// changing the extension of the filename
// filename .. filename to be changed
// maxLength .. maximal length of array filename that should not be exceeded
// newExtension .. newExtension added/replaced  ("vis", "dbg", "rgb" etc.)
void
ChangeFilenameExtension(const string &filename, const string &newExtension,
			string &outputName)
{
  outputName = filename;
  unsigned int pos = (unsigned int)outputName.find_last_of('.');
  
  if ( pos == outputName.length()) {
    // filename has no extension
    outputName.append(".");
  }
  else { // shorten the string
    // If the newExtension is of the zero size
    if (newExtension.size() == 0)
      // remove the former extension including the dot
      // This can be used to remove the last extension completely
      outputName.resize(pos);
    else
      // Keep the last dot before the extension
      outputName.resize(pos+1);
  }

  outputName.append(newExtension);
}

// changing the extension of the filename
// filename .. filename to be changed
// maxLength .. maximal length of array filename that should not be exceeded
// newExtension .. newExtension added/replaced  ("vis", "dbg", "rgb" etc.)
void
ExtractFilenameExtension(const string &filename, string &extension)
{
  extension.assign("");
  unsigned int pos = (unsigned int)filename.find_last_of('.');
  
  int flength = filename.length();
  if ( pos != flength) {
    // extension does exist
    pos++;
    int cnt = flength - pos;
    for (int i = 0; i < cnt; i++, pos++) {
      char c = filename[pos];
      extension.push_back(c);
    } // for i
  }
    
  return;
}


// Skip all the rows in a text file starting from the current character,
// when the current character is a 'mark'. All the following lines
// starting with the 'mark' character are also skipped.
void
EatComments(istream &inpf, char mark)
{
  // current character
  char ch;
  
  // first character to be read
  inpf >> ch;
  
  // loop
  while (ch == mark) {
    inpf.ignore(100000, '\n'); // skip the line completely
    // first character to be read
    if (inpf.eof())
      return; // end of the stream
    else
      inpf >> ch; // read the first character on the line
  }
  
  // return the character back to the stream
  inpf.putback(ch);
}

// Skip all the whitespaces in the input stream
void
EatWhitespace(istream &inpf)
{
  // the current character
  char ch;

  // first character to be read
  inpf >> ch;

  // loop
  while (ch==' ' || ch=='\t' || ch=='\n' || ch=='\f' || ch=='\r' || ch=='\b') {
    if (inpf.eof())
      return; // end of the stream
    else
      inpf >> ch; // read the first character on the line
  }
  
  // return the first non-white character back to the stream to be read by the
  // next operation for reading
  inpf.putback(ch);
}

bool
FindLineWithKeyword(istream &inpf, const string &keyword)
{
  char ch;
  int length = (unsigned int)keyword.size();

  // now we assume that we are at the beggining of a line,
  // otherwise the same string when multiply stated on a line
  // can be found.
  
  for (;;) {
    bool match = true;
    for (int i = 0; i < length; i++) {
      // i-th character on a line to be read
      if (!inpf.get(ch)) {
	match = false;
	inpf.ignore(100000, '\n');
	if (inpf.eof()) {
	  // end of the stream
	  return true; // text not found - end of stream
	}
      }

      if (ch != keyword[i]) {
	inpf.ignore(100000, '\n'); // skip the line completely
	// first character to be read
	match = false; // next line
	break;
      }
    } // for i

    if (match)
      return false; // OK, line starting with 'mark' string found
    // otherwise go to the next line
  }
  return true; // not found
}

// Converts all the possible characters to lowercase
void
ToLowerCase(string &str)
{
  int i;
  int length = (int) str.length();
  for (i = 0; i < length; i++) {
    str[i] = tolower(str[i]);
  }
}

__END_SBRDFLIB_SRC
